"""
===================
Integral transforms
===================

This subpackage gives access to different integral transforms used for
signal processing purposes. The ones currently available are:
- Fourier
- Inverse Fourier
- Hilbert
"""
